import React, { useEffect, useState, useMemo, useRef } from 'react';
import { Select, message } from 'antd';
import { GetSelectItemList, GetFieldValueFromTable, GetFieldValueByTableName } from '../../../../apis/process';
import eventEmitter from '../../../../utils/eventEmitter';
const { Option } = Select;

const RelevanceSelect = props => {
  const { value, onChange, schema, addons, title } = props;
  const {
    $id,
    disabled,
    placeholder,
    fieldParent,
    selectSource,
    dictionary,
    presetValue,
    whereField,
    whereType,
    sourceType,
    fieldTableKey,
    tableName,
    fieldName,
    displayName,
  } = schema;
  const [options, setOptions] = useState([]);
  const [tableData, setTableData] = useState([]);
  const [selectWidth, setSelectWidth] = useState(0);
  const divRef = useRef(null)
  //用于表数据的父子级控件之间联动
  useEffect(() => {
    if(divRef.current){
      const width = divRef.current.offsetWidth;
      setSelectWidth(width)
      console.log('实际宽度：', width, divRef.current);
    }
    if (addons) {
      if (getTargetPath()) {
        const path = getTargetPath();
       
        eventEmitter.on(`${path}_event`, val => dealParentData(val));
      }
      if (((sourceType === '表数据' && !fieldTableKey) || (sourceType === '手动输入' && !fieldParent))  && getChildrenSchemaByKey(title) && presetValue) {
        //表数据类型的父级控件联动子级加载数据
        setTimeout(() => {
          eventEmitter.emit(`${addons?.dataPath}_event`, presetValue);
        }, 100);
      }
    }
    return () => {
      if (getTargetPath()) {
        const path = getTargetPath(fieldTableKey);
        eventEmitter.removeAllListeners(`${path}_event`);
      }
    };
  }, []);

  useEffect(() => {
    if (sourceType === '表数据' && presetValue) {
      setTimeout(() => {
        getTableData();
      }, 100);
    }
    if (sourceType === '手动输入') {
      customData();
    }
    if (addons) {
      addons.setValue(addons.dataPath, presetValue || '');
    } else {
      onChange(presetValue);
    }
  }, [presetValue, sourceType]);

  const selectChange = value => {
    console.log(value, tableData, 'value');
    if (addons) {
      onChange(value || '');

      if (getChildrenSchemaByKey(title)) {
        eventEmitter.emit(`${addons?.dataPath}_event`, value);
      }
    }
  };

  ///处理父级传入的数据-表数据
  const dealParentData = async val => {
   
    if (sourceType === '表数据') {
      const { code, data, msg } = await GetFieldValueByTableName({
        params: {
          tableName: tableName,
          fieldName: `${displayName},${fieldName}`,
        },
        data: [
          {
            field: whereField,
            type: whereType,
            value: val,
          },
        ],
      });
      if (code === 0) {
        if (Array.isArray(data)) {
          setTableData(data.filter(v => v));
          if(data.length > 0){
            let fieldNameData = data[0].find(v => v.fieldName === fieldName);
            onChange(fieldNameData?.fieldValue || '');
          }
        } else {
          setTableData([]);
        }
      } else {
        message.error(msg);
      }
    }
    if (sourceType === '手动输入') {
      console.log(selectSource, addons.dataPath, val, 'selectSource');
      if (selectSource) {
        const jsonObj = JSON.parse(selectSource);
        const arr = jsonObj.filter(v => v.parent.includes(val || '')).map(v => v.value);
        setOptions(arr);
        let v = '';
        if(arr.length > 0){
          if(presetValue && arr.includes(presetValue)){
            v = presetValue;
          }else {
            v = arr[0]
          }
        }else{
          v = ''
        }
        if(!val){
          v = ''
        }
        onChange(v);
        if (getChildrenSchemaByKey(title)) {
          eventEmitter.emit(`${addons?.dataPath}_event`, v);
        }
        // if(!val || (presetValue && !arr.includes(presetValue))){
        //   onChange('');
        // }
      }
      
    }
  };

  const getTargetPath = () => {
    if (((sourceType === '表数据' && !fieldTableKey) || (sourceType === '手动输入' && !fieldParent)) || !addons) return null;
    let targetPath = null;
    for (const k of Object.keys(addons.formData)) {
      for (const s of Object.keys(addons.formData[k])) {
        if(sourceType === '表数据' && fieldTableKey === s){
          targetPath = `${k}.${s}`;
          break;
        }

        if(sourceType === '手动输入' && fieldParent === s){
          targetPath = `${k}.${s}`;
          break;
        }
      }
    }
    return targetPath;
  };

  const getChildrenSchemaByKey = key => {
    let schema = null;
    if (sourceType != '表数据' && sourceType != '手动输入') return null;
    const formData = addons.formData;
    for (const k of Object.keys(formData).filter(f => f != 'relationForm')) {
      for (const s of Object.keys(formData[k])) {
        const path = `${k}.${s}`;
        let _schema = addons?.getSchemaByPath(path);
        if(sourceType === '表数据'){
          if (_schema?.fieldTableKey === fieldTableKey) {
            schema = _schema;
            break;
          }
        }else{
          if (_schema?.fieldParent === fieldParent) {
            schema = _schema;
            break;
          }
        }
      }
    }
    return schema;
  };

  const enums = useMemo(() => {
    switch (sourceType) {
      case '数据字典':
        return Array.isArray(options) ? options : [];
      case '手动输入':
        return Array.isArray(options) ? options : [];
      case '表数据':
        return Array.isArray(tableData) ? tableData : [];
      default:
        return [];
    }
  }, [sourceType, options, tableData]);

  const children = useMemo(() => {
    let children = [];
    if (Array.isArray(enums)) {
      enums.forEach((v, i) => {
        if (sourceType === '数据字典') {
          children.push(
            <Option value={v.nodeValue} key={v}>
              {v.nodeValue}
            </Option>,
          );
        } else if (sourceType === '手动输入') {
          children.push(
            <Option value={v} key={i}>
              {v}
            </Option>,
          );
        } else {
          let displayData = v?.find(v => v.fieldName === displayName);
          let fieldNameData = v?.find(v => v.fieldName === fieldName);
          children.push(
            <Option key={v} value={fieldNameData?.fieldValue}>
              {displayData?.fieldValue}
            </Option>,
          );
        }
      });
    }
    return children;
  }, [sourceType, enums]);

  const onfocus = async () => {
    if (!addons) return;
    if (sourceType === '数据字典') {
      getDictionary();
    } else if (sourceType === '表数据' && !fieldTableKey) {
      getTableData();
    }
  };

  const customData = () => {
    if (selectSource) {
      const jsonObj = JSON.parse(selectSource);
      setOptions(jsonObj.map(v => v.value));
    }
  };

  const getTableData = async () => {
    if (!tableName) {
      return message.info('请选择表名!');
    }
    if (!fieldName) {
      return message.info('请选择存储字段名!');
    }
    if (!displayName) {
      return message.info('请选择展示字段名!');
    }
    const { code, data, msg } = await GetFieldValueByTableName({
      params: {
        tableName: tableName,
        fieldName: `${displayName},${fieldName}`,
      },
      data: [],
    });
    if (code === 0) {
      if (Array.isArray(data)) {
        setTableData(data.filter(v => v));
      } else {
        setTableData([]);
      }
    } else {
      message.error(msg);
    }
  };

  const getDictionary = async () => {
    if (!dictionary) {
      return message.error('缺少数据字典!');
    }
    const { code, data, msg } = await GetSelectItemList({ nodeName: dictionary });
    if (code === 0) {
      if (Array.isArray(data)) {
        let filterKey = null;
        Object.keys(addons.formData).forEach(v => {
          if (addons.formData[v]) {
            Object.keys(addons.formData[v]).forEach(s => {
              if (fieldParent === s) {
                filterKey = addons.formData[v][s];
              }
            });
          }
        });
        if (filterKey) {
          setOptions(data.filter(v => v.nodeName === filterKey));
        } else {
          setOptions(data);
        }
      }
    } else {
      message.error(msg);
    }
  };

  return (
    <div style={{ width: '100%' }} ref={divRef}>
      <Select
        value={value}
        onFocus={onfocus}
        style={{ width: selectWidth > 0 ? `${selectWidth}px` : '100%' }}
        onChange={selectChange}
        disabled={disabled}
        placeholder={disabled ? placeholder || '' : placeholder || '请选择内容'}
        allowClear
      >
        {children}
      </Select>
    </div>
    
  );
};

export default RelevanceSelect;
