import React, { useEffect, useRef, useState } from 'react';
import { Button, Select, message, Modal, Input, Space } from 'antd';
import { LoadTableV2, LoadTableFields, ReloadTableFields } from '../../../../../apis/process';
import { isArray, isObject } from '../../../../../utils';

const { TextArea } = Input;
const DataSelect = props => {
  console.log(props, '关联选择数据导入');
  const { value, onChange, addons } = props;
  const { tableName, tableNameParent, fieldParent } = addons.formData;
  const [options, setOptions] = useState([]);
  const [parentOptions, setParentOptions] = useState([]);
  const [showType, setShowType] = useState('');
  const [visible, setVisible] = useState(false);
  const [text, setText] = useState('');
  const [pSelectVal, setPSelectVal] = useState('')
  const [selectVal, setSelectVal] = useState([])
  let currentVals = useRef([])
  let parentVals = useRef([])

  useEffect(()=>{
    if(value){
      const objList = JSON.parse(value)
      currentVals.current = objList
      setOptions(objList.map(v=> {
        return {
          label: v.value,
          value: v.value
        }
      }))
      setText(objList.map(v=> v.value).join('\n'))
    }
  }, [value])

  useEffect(()=>{
    if(fieldParent){
      console.log(getParentSchema(fieldParent), 'getParentSchema(fieldParent)') 
      getParentSource(fieldParent)
    }
  }, [fieldParent])

  const getParentSource = (fieldName)=>{
    const schema = getParentSchema(fieldName)
    if(schema){
      const { selectSource } = schema;
      const arr= JSON.parse(selectSource)
      parentVals.current = arr
      if(isArray(arr)){
        setParentOptions(arr.map(v=> {
          return {
            label: v.value,
            value: v.value
          }
        }))
      }
    }
  }
    const getParentSchema = (fieldName) => {
      if(!window.designer) return null;
      return window.designer.find(v=> v.fieldName === fieldName)
    }

  const handleClick = (type)=>{
    if(type === 'config' && !fieldParent){
      message.warn('请先选择父字段名！')
      return;
    }
    setShowType(type)
    setVisible(true)
  }

  const textInput = (e)=>{
    setText(e.target.value)
  }

  const onOk = ()=>{
    if(showType === 'import'){
      const currentArr = text.split('\n')
      const arr = currentArr.filter((item, index) => currentArr.indexOf(item) === index && item);
      if(currentVals.current.length > 0){//如果已有值则追加
        let objList= currentVals.current
        objList = objList.filter(v=> arr.includes(v.value))
        objList = [
          ...objList,
          ...(arr.filter(v=> !objList.map(x=> x.value).includes(v)).map(v=> {
            return {
              value: v,
              parent: []
            }
          }))
        ]
        currentVals.current = objList
        onChange(JSON.stringify(objList))
      }else{
        const arr1 = arr.map(v=> {
          return {
            value: v,
            parent: []
          }
        })
        currentVals.current = arr1
        onChange(JSON.stringify(arr1))
      }
    }else{
      console.log(currentVals, 'currentVals')
      onChange(JSON.stringify(currentVals.current))
    }

    setVisible(false)
    setPSelectVal('')
    setSelectVal([])
  }

  const handleSelect = (e)=>{
    if(Array.isArray(e)){//子级
      setSelectVal(e)
      if(e.length > 0){
        currentVals.current.forEach(v=> {
          if(v.parent.includes(pSelectVal)){//找到关联了父级的子选项
            if(!e.includes(v.value)){//如果子选项不存在则剔除父级
              v.parent = v.parent.filter(x=> x !== pSelectVal)
            }
          }
          if(e.includes(v.value)){
            if(!v.parent.includes(pSelectVal)){
              v.parent.push(pSelectVal)
            }
          }
        })
      }else{
        currentVals.current.forEach(v=> {
          if(v.parent.includes(pSelectVal)){
            v.parent = v.parent.filter(x=> x !== pSelectVal)
          }
        })
      }
    }else{
      setPSelectVal(e)
      const vals = currentVals.current.filter(v=> v.parent.includes(e)).map(v=> v.value)
      setSelectVal(vals)
    }
  }
  
  const handleClose = ()=>{
    setVisible(false)
    setPSelectVal('')
    setSelectVal([])
  }
  return (
    <>
      <Select options={options} placeholder={options.length ? '已配置' : ''}></Select>
      <Button type={'link'} onClick={() => handleClick('import')}>
        数据导入
      </Button>
      <Button type={'link'} onClick={() => handleClick('config')}>联动配置</Button>
      <Modal title={showType === 'import' ? '数据导入' : '父子级联动'} visible={visible} onCancel={handleClose} onOk={onOk} destroyOnClose>
        {showType === 'import' && (
          <div>
            <p>数据源批量导入，一条数据独占一行</p>
            <TextArea value={text} rows={6} onChange={textInput}></TextArea>
          </div>
        )}

        {showType === 'config' && (
          <Space>
            <span>父级选项：</span>
            <Select value={pSelectVal} onChange={handleSelect} options={parentOptions} style={{ width: '120px' }}></Select>
            <span>子级选项：</span>
            <Select value={selectVal} mode={'multiple'} maxTagCount={2} onChange={handleSelect}  style={{ width: '160px' }} options={pSelectVal.length && options}></Select>
          </Space>
        )}
      </Modal>
    </>
  );
};

export default DataSelect;
